@@ -29,14 +29,17 @@
 char buf[BUFSZ];
 
 #define N_CHANS 256
-unsigned int chanlist[N_CHANS];
+static unsigned int chanlist[N_CHANS];
+static comedi_range * range_info[N_CHANS];
+static lsampl_t maxdata[N_CHANS];
 
 
-int prepare_cmd_lib(comedi_t *dev,int subdevice,comedi_cmd *cmd);
-int prepare_cmd(comedi_t *dev,int subdevice,comedi_cmd *cmd);
+int prepare_cmd_lib(comedi_t *dev, int subdevice, int n_scan, int n_chan, unsigned period_nanosec, comedi_cmd *cmd);
 
 void do_cmd(comedi_t *dev,comedi_cmd *cmd);
 
+void print_datum(lsampl_t raw, int channel_index, short physical);
+
 char *cmdtest_messages[]={
 	"success",
 	"invalid source",
@@ -54,42 +57,51 @@ int main(int argc, char *argv[])
 	int total=0;
 	int i;
 	struct timeval start,end;
+	int subdev_flags;
+	lsampl_t raw;
+	struct parsed_options options;
 
-	parse_options(argc,argv);
+	init_parsed_options(&options);
+	parse_options(&options, argc, argv);
 
-	/* The following global variables used in this demo are
-	 * defined in common.c, and can be modified by command line
+	/* The following variables used in this demo
+	 * can be modified by command line
 	 * options.  When modifying this demo, you may want to
 	 * change them here. */
-	//filename = "/dev/comedi0";
-	//subdevice = 0;
-	//channel = 0;
-	//range = 0;
-	//aref = AREF_GROUND;
-	//n_chan = 4;
-	//n_scan = 1000;
-	//freq = 1000.0;
+	//options.filename = "/dev/comedi0";
+	//options.subdevice = 0;
+	//options.channel = 0;
+	//options.range = 0;
+	//options.aref = AREF_GROUND;
+	//options.n_chan = 4;
+	//options.n_scan = 1000;
+	//options.freq = 1000.0;
 
 	/* open the device */
-	dev = comedi_open(filename);
+	dev = comedi_open(options.filename);
 	if(!dev){
-		comedi_perror(filename);
+		comedi_perror(options.filename);
 		exit(1);
 	}
 
+	// Print numbers for clipped inputs
+	comedi_set_global_oor_behavior(COMEDI_OOR_NUMBER);
+
 	/* Set up channel list */
-	for(i=0;i<n_chan;i++){
-		chanlist[i]=CR_PACK(channel+i,range,aref);
+	for(i = 0; i < options.n_chan; i++){
+		chanlist[i] = CR_PACK(options.channel + i, options.range, options.aref);
+		range_info[i] = comedi_get_range(dev, options.subdevice, options.channel, options.range);
+		maxdata[i] = comedi_get_maxdata(dev, options.subdevice, options.channel);
 	}
 
 	/* prepare_cmd_lib() uses a Comedilib routine to find a
 	 * good command for the device.  prepare_cmd() explicitly
 	 * creates a command, which may not work for your device. */
-	prepare_cmd_lib(dev,subdevice,cmd);
-	//prepare_cmd(dev,subdevice,cmd);
-	
-	fprintf(stderr,"command before testing:\n");
-	dump_cmd(stderr,cmd);
+	prepare_cmd_lib(dev, options.subdevice, options.n_scan, options.n_chan, 1e9 / options.freq, cmd);
+	//prepare_cmd(dev, options.subdevice, options.n_scan, options.n_chan, 1e9 / options.freq, cmd);
+
+	fprintf(stderr, "command before testing:\n");
+	dump_cmd(stderr, cmd);
 
 	/* comedi_command_test() tests a command to see if the
 	 * trigger sources and arguments are valid for the subdevice.
@@ -101,59 +113,69 @@ int main(int argc, char *argv[])
 	 * can test it multiple times until it passes.  Typically,
 	 * if you can't get a valid command in two tests, the original
 	 * command wasn't specified very well. */
-	ret = comedi_command_test(dev,cmd);
-	if(ret<0){
+	ret = comedi_command_test(dev, cmd);
+	if(ret < 0){
 		comedi_perror("comedi_command_test");
-		if(errno==EIO){
+		if(errno == EIO){
 			fprintf(stderr,"Ummm... this subdevice doesn't support commands\n");
 		}
 		exit(1);
 	}
-	fprintf(stderr,"first test returned %d (%s)\n",ret,
+	fprintf(stderr,"first test returned %d (%s)\n", ret,
 			cmdtest_messages[ret]);
-	dump_cmd(stderr,cmd);
+	dump_cmd(stderr, cmd);
 
-	ret = comedi_command_test(dev,cmd);
-	if(ret<0){
+	ret = comedi_command_test(dev, cmd);
+	if(ret < 0){
 		comedi_perror("comedi_command_test");
 		exit(1);
 	}
-	fprintf(stderr,"second test returned %d (%s)\n",ret,
+	fprintf(stderr,"second test returned %d (%s)\n", ret,
 			cmdtest_messages[ret]);
 	if(ret!=0){
-		dump_cmd(stderr,cmd);
-		fprintf(stderr,"Error preparing command\n");
+		dump_cmd(stderr, cmd);
+		fprintf(stderr, "Error preparing command\n");
 		exit(1);
 	}
 
 	/* this is only for informational purposes */
-	gettimeofday(&start,NULL);
-	fprintf(stderr,"start time: %ld.%06ld\n",start.tv_sec,start.tv_usec);
+	gettimeofday(&start, NULL);
+	fprintf(stderr,"start time: %ld.%06ld\n", start.tv_sec, start.tv_usec);
 
 	/* start the command */
-	ret=comedi_command(dev,cmd);
-	if(ret<0){
+	ret = comedi_command(dev, cmd);
+	if(ret < 0){
 		comedi_perror("comedi_command");
 		exit(1);
 	}
-
+	subdev_flags = comedi_get_subdevice_flags(dev, options.subdevice);
 	while(1){
-		ret=read(comedi_fileno(dev),buf,BUFSZ);
-		if(ret<0){
+		ret = read(comedi_fileno(dev),buf,BUFSZ);
+		if(ret < 0){
 			/* some error occurred */
 			perror("read");
 			break;
-		}else if(ret==0){
+		}else if(ret == 0){
 			/* reached stop condition */
 			break;
 		}else{
 			static int col = 0;
-			total+=ret;
-			if(verbose)fprintf(stderr,"read %d %d\n",ret,total);
-			for(i=0;i<ret/2;i++){
-				printf("%d ",((sampl_t *)buf)[i]);
+			int bytes_per_sample;
+			total += ret;
+			if(options.verbose)fprintf(stderr, "read %d %d\n", ret, total);
+			if(subdev_flags & SDF_LSAMPL)
+				bytes_per_sample = sizeof(lsampl_t);
+			else
+				bytes_per_sample = sizeof(sampl_t);
+			for(i = 0; i < ret / bytes_per_sample; i++){
+				if(subdev_flags & SDF_LSAMPL) {
+					raw = ((lsampl_t *)buf)[i];
+				} else {
+					raw = ((sampl_t *)buf)[i];
+				}
+				print_datum(raw, col, options.physical);
 				col++;
-				if(col==n_chan){
+				if(col == options.n_chan){
 					printf("\n");
 					col=0;
 				}
@@ -163,15 +185,15 @@ int main(int argc, char *argv[])
 
 	/* this is only for informational purposes */
 	gettimeofday(&end,NULL);
-	fprintf(stderr,"end time: %ld.%06ld\n",end.tv_sec,end.tv_usec);
+	fprintf(stderr,"end time: %ld.%06ld\n", end.tv_sec, end.tv_usec);
 
-	end.tv_sec-=start.tv_sec;
-	if(end.tv_usec<start.tv_usec){
+	end.tv_sec -= start.tv_sec;
+	if(end.tv_usec < start.tv_usec){
 		end.tv_sec--;
-		end.tv_usec+=1000000;
+		end.tv_usec += 1000000;
 	}
-	end.tv_usec-=start.tv_usec;
-	fprintf(stderr,"time: %ld.%06ld\n",end.tv_sec,end.tv_usec);
+	end.tv_usec -= start.tv_usec;
+	fprintf(stderr,"time: %ld.%06ld\n", end.tv_sec, end.tv_usec);
 
 	return 0;
 }
@@ -180,7 +202,7 @@ int main(int argc, char *argv[])
  * This prepares a command in a pretty generic way.  We ask the
  * library to create a stock command that supports periodic
  * sampling of data, then modify the parts we want. */
-int prepare_cmd_lib(comedi_t *dev,int subdevice,comedi_cmd *cmd)
+int prepare_cmd_lib(comedi_t *dev, int subdevice, int n_scan, int n_chan, unsigned scan_period_nanosec, comedi_cmd *cmd)
 {
 	int ret;
 
@@ -189,18 +211,16 @@ int prepare_cmd_lib(comedi_t *dev,int subdevice,comedi_cmd *cmd)
 	/* This comedilib function will get us a generic timed
 	 * command for a particular board.  If it returns -1,
 	 * that's bad. */
-	ret = comedi_get_cmd_generic_timed(dev,subdevice,cmd,1e9/freq);
+	ret = comedi_get_cmd_generic_timed(dev, subdevice, cmd, n_chan, scan_period_nanosec);
 	if(ret<0){
 		printf("comedi_get_cmd_generic_timed failed\n");
 		return ret;
 	}
 
 	/* Modify parts of the command */
-	cmd->chanlist		= chanlist;
-	cmd->chanlist_len	= n_chan;
-
-	cmd->scan_end_arg = n_chan;
-	if(cmd->stop_src==TRIG_COUNT)cmd->stop_arg = n_scan;
+	cmd->chanlist = chanlist;
+	cmd->chanlist_len = n_chan;
+	if(cmd->stop_src == TRIG_COUNT) cmd->stop_arg = n_scan;
 
 	return 0;
 }
@@ -209,7 +229,7 @@ int prepare_cmd_lib(comedi_t *dev,int subdevice,comedi_cmd *cmd)
  * Set up a command by hand.  This will not work on some devices.
  * There is no single command that will work on all devices.
  */
-int prepare_cmd(comedi_t *dev,int subdevice,comedi_cmd *cmd)
+int prepare_cmd(comedi_t *dev, int subdevice, int n_scan, int n_chan, unsigned period_nanosec, comedi_cmd *cmd)
 {
 	memset(cmd,0,sizeof(*cmd));
 
@@ -221,7 +241,7 @@ int prepare_cmd(comedi_t *dev,int subdevice,comedi_cmd *cmd)
 
 	/* Wake up at the end of every scan */
 	//cmd->flags |= TRIG_WAKE_EOS;
-	
+
 	/* Use a real-time interrupt, if available */
 	//cmd->flags |= TRIG_RT;
 
@@ -262,7 +282,7 @@ int prepare_cmd(comedi_t *dev,int subdevice,comedi_cmd *cmd)
 	 * by the device, but it will be adjusted to the nearest supported
 	 * value by comedi_command_test(). */
 	cmd->scan_begin_src =	TRIG_TIMER;
-	cmd->scan_begin_arg =	1e9/freq;		/* in ns */
+	cmd->scan_begin_arg = period_nanosec;		/* in ns */
 
 	/* The timing between each sample in a scan is controlled by convert.
 	 * TRIG_TIMER:   Conversion events occur periodically.
@@ -304,4 +324,12 @@ int prepare_cmd(comedi_t *dev,int subdevice,comedi_cmd *cmd)
 	return 0;
 }
 
-
+void print_datum(lsampl_t raw, int channel_index, short physical) {
+	double physical_value;
+	if(!physical) {
+		printf("%d ",raw);
+	} else {
+		physical_value = comedi_to_phys(raw, range_info[channel_index], maxdata[channel_index]);
+		printf("%#8.6g ",physical_value);
+	}
+}
