@@ -21,6 +21,7 @@
     USA.
 */
 
+#include <assert.h>
 #include <stdio.h>
 #include <math.h>
 #include <stdlib.h>
@@ -49,18 +50,31 @@ int get_subdevices(comedi_t *it)
 	int ret;
 	comedi_subdinfo *s;
 	comedi_chaninfo ci;
-	subdevice *r;
+	subdevice *r = NULL;
 
-	s=malloc(sizeof(comedi_subdinfo)*it->n_subdevices);
-	debug_ptr(s);
+	s = calloc(it->n_subdevices, sizeof(comedi_subdinfo));
+	if(s == NULL)
+	{
+		debug_ptr(s);
+		libc_error();
+		goto cleanup;
+	}
 
-	ret = comedi_ioctl(it->fd, COMEDI_SUBDINFO, (unsigned long)s);
-	debug_int(ret);
+	ret = comedi_ioctl(it->fd, COMEDI_SUBDINFO, s);
+	if(ret < 0)
+	{
+		debug_int(ret);
+		goto cleanup;
+	}
 
-	r=it->subdevices=realloc(it->subdevices,
-		sizeof(subdevice)*it->n_subdevices);
-	debug_ptr(r);
-	memset(r,0,sizeof(subdevice)*it->n_subdevices);
+	assert(it->subdevices == NULL);
+	r = it->subdevices = calloc(it->n_subdevices, sizeof(subdevice));
+	if(r == NULL)
+	{
+		debug_ptr(r);
+		libc_error();
+		goto cleanup;
+	}
 
 	it->has_insnlist_ioctl = do_test_for_insnlist(it);
 	it->has_insn_ioctl = do_test_for_insn(it);
@@ -76,45 +90,111 @@ int get_subdevices(comedi_t *it)
 		r[i].range_type = s[i].range_type;
 
 		if(r[i].subd_flags&SDF_FLAGS){
-			r[i].flags_list=malloc(sizeof(*r[i].flags_list)*r[i].n_chan);
-			debug_ptr(r[i].flags_list);
+			r[i].flags_list = calloc(r[i].n_chan, sizeof(*r[i].flags_list));
+			if(r[i].flags_list == NULL){
+				debug_ptr(r[i].flags_list);
+				libc_error();
+				goto cleanup;
+			}
+
 		}
 		if(r[i].subd_flags&SDF_MAXDATA){
-			r[i].maxdata_list=malloc(sizeof(*r[i].maxdata_list)*r[i].n_chan);
-			debug_ptr(r[i].maxdata_list);
+			r[i].maxdata_list = calloc(r[i].n_chan, sizeof(*r[i].maxdata_list));
+			if(r[i].maxdata_list == NULL){
+				debug_ptr(r[i].maxdata_list);
+				libc_error();
+				goto cleanup;
+			}
 		}
 		if(r[i].subd_flags&SDF_RANGETYPE){
-			r[i].range_type_list=malloc(sizeof(*r[i].range_type_list)*r[i].n_chan);
-			debug_ptr(r[i].range_type_list);
+			r[i].range_type_list = calloc(r[i].n_chan, sizeof(*r[i].range_type_list));
+			if(r[i].range_type_list == NULL){
+				debug_ptr(r[i].range_type_list);
+				libc_error();
+				goto cleanup;
+			}
 		}
 		ci.subdev = i;
 		ci.flaglist = r[i].flags_list;
 		ci.rangelist = r[i].range_type_list;
 		ci.maxdata_list = r[i].maxdata_list;
-		ret = comedi_ioctl(it->fd, COMEDI_CHANINFO, (unsigned long)&ci);
-		debug_int(ret);
+		ret = comedi_ioctl(it->fd, COMEDI_CHANINFO, &ci);
+		if(ret < 0){
+			debug_int(ret);
+			goto cleanup;
+		}
 
 		if(r[i].subd_flags&SDF_RANGETYPE){
-			r[i].rangeinfo_list=malloc(sizeof(*r[i].rangeinfo_list)*r[i].n_chan);
-			debug_ptr(r[i].rangeinfo_list);
+			r[i].rangeinfo_list=calloc(r[i].n_chan, sizeof(*r[i].rangeinfo_list));
+			if(r[i].rangeinfo_list == NULL){
+				debug_ptr(r[i].rangeinfo_list);
+				libc_error();
+				goto cleanup;
+			}
 			for(j=0;j<r[i].n_chan;j++){
 				r[i].rangeinfo_list[j]=get_rangeinfo(it->fd,r[i].range_type_list[j]);
+				if(r[i].rangeinfo_list[j] == NULL)
+					goto cleanup;
 			}
 		}else{
 			r[i].rangeinfo=get_rangeinfo(it->fd,r[i].range_type);
+			if(r[i].rangeinfo == NULL)
+				goto cleanup;
 		}
 
 		r[i].has_cmd = do_test_for_cmd(it,i);
-		if(it->has_insnlist_ioctl){
-			r[i].has_insn_bits = do_test_for_insn_bits(it,i);
-		}else{
+		switch(s[i].insn_bits_support)
+		{
+		case COMEDI_UNKNOWN_SUPPORT:
+			if(it->has_insnlist_ioctl){
+				r[i].has_insn_bits = do_test_for_insn_bits(it,i);
+			}else{
+				r[i].has_insn_bits = 0;
+			}
+			break;
+		case COMEDI_SUPPORTED:
+			r[i].has_insn_bits = 1;
+			break;
+		case COMEDI_UNSUPPORTED:
 			r[i].has_insn_bits = 0;
+			break;
+		default:
+			assert(0);
 		}
 	}
 
 	free(s);
 
 	return 0;
+
+cleanup:
+
+	if(s)
+		free(s);
+
+	if(r){
+		for(i=0;i<it->n_subdevices;i++){
+			if(r[i].flags_list)
+				free(r[i].flags_list);
+			if(r[i].maxdata_list)
+				free(r[i].maxdata_list);
+			if(r[i].range_type_list)
+				free(r[i].range_type_list);
+			if(r[i].rangeinfo_list){
+				for(j=0;j<r[i].n_chan;j++){
+					if(r[i].rangeinfo_list[j])
+						free(r[i].rangeinfo_list[j]);
+				}
+			}else{
+				if(r[i].rangeinfo)
+					free(r[i].rangeinfo);
+			}
+		}
+		free(r);
+		it->subdevices = NULL;
+	}
+
+	return -1;
 }
 
 comedi_range *get_rangeinfo(int fd,unsigned int range_type)
@@ -125,14 +205,31 @@ comedi_range *get_rangeinfo(int fd,unsigned int range_type)
 	int ret;
 	int i;
 
-	kr=malloc(sizeof(comedi_krange)*RANGE_LENGTH(range_type));
-	r=malloc(sizeof(comedi_range)*RANGE_LENGTH(range_type));
+	kr = calloc(RANGE_LENGTH(range_type), sizeof(comedi_krange));
+	if(kr == NULL)
+	{
+		debug_ptr(kr);
+		libc_error();
+		return NULL;
+	}
+	r = calloc(RANGE_LENGTH(range_type), sizeof(comedi_range));
+	if(r == NULL)
+	{
+		debug_ptr(r);
+		libc_error();
+		free(kr);
+		return NULL;
+	}
 
+	memset(&ri, 0, sizeof(ri));
 	ri.range_type = range_type;
 	ri.range_ptr = kr;
-	ret = comedi_ioctl(fd, COMEDI_RANGEINFO, (unsigned long)&ri);
+	ret = comedi_ioctl(fd, COMEDI_RANGEINFO, &ri);
 	if(ret<0){
 		fprintf(stderr,"ioctl(%d,COMEDI_RANGEINFO,0x%08x,%p)\n",fd,range_type,kr);
+		free(r);
+		free(kr);
+		return NULL;
 	}
 
 	for(i=0;i<RANGE_LENGTH(range_type);i++){
@@ -168,7 +265,7 @@ static int do_test_for_cmd(comedi_t *dev,unsigned int subdevice)
 		it.scan_end_src = TRIG_ANY;
 		it.stop_src = TRIG_ANY;
 
-		ret = comedi_ioctl(dev->fd, COMEDI_CMDTEST, (unsigned long)&it);
+		ret = comedi_ioctl(dev->fd, COMEDI_CMDTEST, &it);
 
 		if(ret<0 && errno==EIO){
 			return 0;
@@ -188,20 +285,20 @@ static int do_test_for_insnlist(comedi_t *dev)
 	lsampl_t data[2];
 	int ret;
 
-	memset(&insn,0,sizeof(insn));
-
 	il.n_insns = 1;
 	il.insns = &insn;
 
+	memset(&insn,0,sizeof(insn));
 	insn.insn = INSN_GTOD;
 	insn.n = 2;
 	insn.data = data;
+	memset(insn.data, 0, insn.n * sizeof(insn.data[0]));
 
-	ret = comedi_ioctl(dev->fd, COMEDI_INSNLIST, (unsigned long)&il);
+	ret = comedi_ioctl(dev->fd, COMEDI_INSNLIST, &il);
 
 	if(ret<0){
 		if(errno!=EIO){
-			fprintf(stderr,"BUG in do_test_for_insn()\n");
+			fprintf(stderr,"BUG in do_test_for_insnlist()\n");
 		}
 		return 0;
 	}
@@ -212,20 +309,16 @@ static int do_test_for_insnlist(comedi_t *dev)
 static int do_test_for_insn(comedi_t *dev)
 {
 	comedi_insn insn;
-	comedi_insnlist il;
 	lsampl_t data[2];
 	int ret;
 
 	memset(&insn,0,sizeof(insn));
-
-	il.n_insns = 1;
-	il.insns = &insn;
-
 	insn.insn = INSN_GTOD;
 	insn.n = 2;
 	insn.data = data;
+	memset(insn.data, 0, insn.n * sizeof(insn.data[0]));
 
-	ret = comedi_ioctl(dev->fd, COMEDI_INSN, (unsigned long)&insn);
+	ret = comedi_ioctl(dev->fd, COMEDI_INSN, &insn);
 
 	if(ret<0){
 		if(errno!=EIO){
@@ -255,11 +348,9 @@ static int do_test_for_insn_bits(comedi_t *dev,unsigned int subdevice)
 	insn.n = 2;
 	insn.data = data;
 	insn.subdev = subdevice;
+	memset(data, 0, insn.n * sizeof(data[0]));
 
-	data[0]=0;
-	data[1]=0;
-
-	ret = comedi_do_insnlist(dev,&il);
+	ret = comedi_ioctl(dev->fd, COMEDI_INSNLIST, &il);
 
 	if(ret<0 && (errno==EINVAL || errno==EIO)){
 		return 0;
