@@ -85,6 +85,8 @@ void internal_error(int error_number);
 typedef struct subdevice_struct subdevice;
 typedef struct device_struct device;
 
+typedef struct calib_yyparse_private calib_yyparse_private_t;
+
 struct comedi_t_struct{
 	int magic;
 
@@ -121,8 +123,6 @@ struct subdevice_struct{
 
 	int cmd_mask_errno;
 	comedi_cmd *cmd_mask;
-	int cmd_timed_errno;
-	comedi_cmd *cmd_timed;
 };
 
 #define comedi_ioctl _comedi_ioctl
@@ -146,8 +146,6 @@ int valid_chan(comedi_t *it,unsigned int subdevice,unsigned int chan);
 int comedi_get_rangetype(comedi_t *it,unsigned int subdevice,unsigned int chan);
 
 #define YY_DECL int calib_yylex(YYSTYPE *calib_lvalp, yyscan_t yyscanner)
-void calib_yyerror(char *s);
-int calib_yyparse(void *parse_arg);
 
 #endif
 
