@@ -111,7 +111,7 @@ int _comedi_find_range(comedi_t *it,unsigned int subd,unsigned int chan,unsigned
 	best_ptr=NULL;
 	for(i=0;i<RANGE_LENGTH(range_type);i++){
 		range_ptr=comedi_get_range(it,subd,chan,i);
-		if(range_ptr->min<=min && range_ptr->max>=max){
+		if(range_ptr->unit==unit && range_ptr->min<=min && range_ptr->max>=max){
 			if(best<0 || (range_ptr->max-range_ptr->min) < 
 			   (best_ptr->max-best_ptr->min)){
 				best=i;
@@ -136,6 +136,7 @@ int _comedi_get_n_ranges(comedi_t *it,unsigned int subd,unsigned int chan)
 EXPORT_ALIAS_DEFAULT(_comedi_range_is_chan_specific,comedi_range_is_chan_specific,0.7.18);
 int _comedi_range_is_chan_specific(comedi_t *it,unsigned int subd)
 {
+	if(!valid_subd(it,subd)) return -1;
 	return (it->subdevices[subd].subd_flags&SDF_RANGETYPE)?1:0;
 }
 
