@@ -21,7 +21,7 @@ int test_cmd_fifo_depth_check(void)
 	int len;
 	unsigned int flags = comedi_get_subdevice_flags(device,subdevice);
 
-	if(!(flags&SDF_CMD) || flags&SDF_WRITEABLE){
+	if(!(flags&SDF_CMD) || (comedi_get_read_subdevice(device)!=subdevice)){
 		printf("not applicable\n");
 		return 0;
 	}
